package com.persagy.common;

import lombok.Data;

import java.util.Objects;

/**
 * 错误码对象
 * <p>
 * 全局错误码，参见 {@link ErrorCodeConstants}
 * 错误码设计成对象的原因，为未来的 i18 国际化做准备
 * <p>
 */
@Data
public class ErrorCode {

    /**
     * 错误码
     */
    private final String code;
    /**
     * 错误提示
     */
    private final String message;

    public ErrorCode(Integer code, String message) {
        this.code = String.valueOf(code);
        this.message = message;
    }

    public ErrorCode(String code, String message) {
        this.code = code;
        this.message = message;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof ErrorCode)) {
            return false;
        }
        ErrorCode errorCode = (ErrorCode) o;
        return code.equals(errorCode.code) && message.equals(errorCode.message);
    }

    @Override
    public int hashCode() {
        return Objects.hash(code, message);
    }
}
